"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const iterall_1 = require("iterall");
const nats_pubsub_1 = require("../nats-pubsub");
const graphql_1 = require("graphql");
const graphql_subscriptions_1 = require("graphql-subscriptions");
const subscription_1 = require("graphql/subscription");
const logger_1 = require("./logger");
const FIRST_EVENT = 'FIRST_EVENT';
const defaultFilter = (payload) => true;
function buildSchema(iterator, filterFn = defaultFilter) {
    return new graphql_1.GraphQLSchema({
        query: new graphql_1.GraphQLObjectType({
            name: 'Query',
            fields: {
                testString: {
                    type: graphql_1.GraphQLString,
                    resolve: function (_, args) {
                        return 'works';
                    },
                },
            },
        }),
        subscription: new graphql_1.GraphQLObjectType({
            name: 'Subscription',
            fields: {
                testSubscription: {
                    type: graphql_1.GraphQLString,
                    subscribe: graphql_subscriptions_1.withFilter(() => iterator, filterFn),
                    resolve: root => {
                        return 'FIRST_EVENT';
                    },
                },
            },
        }),
    });
}
describe('GraphQL-JS asyncIterator', () => {
    it('should allow subscriptions', () => {
        const query = graphql_1.parse(`
        subscription S1 {
            testSubscription
            }
    `);
        const pubsub = new nats_pubsub_1.NatsPubSub({ logger: logger_1.logger });
        const origIterator = pubsub.asyncIterator(FIRST_EVENT);
        const orig2Iterator = pubsub.asyncIterator('TEST123');
        const schema = buildSchema(origIterator);
        const results = subscription_1.subscribe(schema, query);
        const payload1 = results.next();
        expect(iterall_1.isAsyncIterable(results)).toBeTruthy();
        const r = payload1.then(res => {
            expect(res.value.data.testSubscription).toEqual('FIRST_EVENT');
        });
        pubsub.publish(FIRST_EVENT, { test: { file: true } });
        return r;
    });
    it('should detect when the payload is done when filtering', (done) => {
        const query = graphql_1.parse(`
        subscription S1 {
            testSubscription
        }
        `);
        const pubsub = new nats_pubsub_1.NatsPubSub({ logger: logger_1.logger });
        const origIterator = pubsub.asyncIterator(FIRST_EVENT);
        let counter = 0;
        const filterFn = () => {
            counter++;
            if (counter > 10) {
                const e = new Error('Infinite loop detected');
                done(e);
                throw e;
            }
            return false;
        };
        const schema = buildSchema(origIterator, filterFn);
        const results = subscription_1.subscribe(schema, query);
        expect(iterall_1.isAsyncIterable(results)).toBeTruthy();
        results.next();
        results.return();
        pubsub.publish(FIRST_EVENT, {});
        setTimeout(_ => {
            done();
        }, 500);
    });
    it('should clear event handlers', () => {
        const query = graphql_1.parse(`
        subscription S1 {
            testSubscription
        }
        `);
        const pubsub = new nats_pubsub_1.NatsPubSub({ logger: logger_1.logger });
        const origIterator = pubsub.asyncIterator(FIRST_EVENT);
        const returnSpy = jest.spyOn(origIterator, 'return');
        const schema = buildSchema(origIterator);
        const results = subscription_1.subscribe(schema, query);
        const end = results.return();
        const r = end.then(res => {
            expect(returnSpy).toBeCalled();
        });
        pubsub.publish(FIRST_EVENT, {});
        return r;
    });
});
//# sourceMappingURL=async-iterator-subscription.test.js.map